package org.jboss.security.ssl;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.UnknownHostException;
import java.util.Arrays;
import javax.naming.InitialContext;
import javax.net.ServerSocketFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;
import org.jboss.logging.Logger;
import org.jboss.security.SecurityDomain;
import org.jboss.security.ssl.Context;

public class DomainServerSocketFactory extends SSLServerSocketFactory {

   private static Logger log = Logger.getLogger(DomainServerSocketFactory.class);
   private transient SecurityDomain securityDomain;
   private transient InetAddress bindAddress;
   private transient SSLContext sslCtx = null;
   private boolean wantsClientAuth = true;
   private boolean needsClientAuth = false;
   private String[] ciperSuites;
   private String[] protocols;


   public DomainServerSocketFactory() {
   }

   public DomainServerSocketFactory(SecurityDomain securityDomain) throws IOException {
      if(securityDomain == null) {
         throw new IOException("The securityDomain may not be null");
      } else {
         this.securityDomain = securityDomain;
      }
   }

   public String getBindAddress() {
      String address = null;
      if(this.bindAddress != null) {
         address = this.bindAddress.getHostAddress();
      }

      return address;
   }

   public void setBindAddress(String host) throws UnknownHostException {
      this.bindAddress = InetAddress.getByName(host);
   }

   public SecurityDomain getSecurityDomain() {
      return this.securityDomain;
   }

   public void setSecurityDomain(SecurityDomain securityDomain) {
      this.securityDomain = securityDomain;
   }

   public boolean isWantsClientAuth() {
      return this.wantsClientAuth;
   }

   public void setWantsClientAuth(boolean wantsClientAuth) {
      this.wantsClientAuth = wantsClientAuth;
   }

   public boolean isNeedsClientAuth() {
      return this.needsClientAuth;
   }

   public void setNeedsClientAuth(boolean needsClientAuth) {
      this.needsClientAuth = needsClientAuth;
   }

   public String[] getCiperSuites() {
      return this.ciperSuites;
   }

   public void setCiperSuites(String[] ciperSuites) {
      this.ciperSuites = ciperSuites;
   }

   public String[] getProtocols() {
      return this.protocols;
   }

   public void setProtocols(String[] protocols) {
      this.protocols = protocols;
   }

   public ServerSocket createServerSocket(int port) throws IOException {
      return this.createServerSocket(port, 50, this.bindAddress);
   }

   public ServerSocket createServerSocket(int port, int backlog) throws IOException {
      return this.createServerSocket(port, backlog, this.bindAddress);
   }

   public ServerSocket createServerSocket(int port, int backlog, InetAddress ifAddress) throws IOException {
      this.initSSLContext();
      SSLServerSocketFactory factory = this.sslCtx.getServerSocketFactory();
      SSLServerSocket socket = (SSLServerSocket)factory.createServerSocket(port, backlog, ifAddress);
      if(log.isTraceEnabled()) {
         String[] supportedProtocols = socket.getSupportedProtocols();
         log.debug("Supported protocols: " + Arrays.asList(supportedProtocols));
         String[] supportedCipherSuites = socket.getSupportedCipherSuites();
         log.debug("Supported CipherSuites: " + Arrays.asList(supportedCipherSuites));
      }

      socket.setNeedClientAuth(this.needsClientAuth);
      socket.setWantClientAuth(this.wantsClientAuth);
      if(this.protocols != null) {
         socket.setEnabledProtocols(this.protocols);
      }

      if(this.ciperSuites != null) {
         socket.setEnabledCipherSuites(this.ciperSuites);
      }

      return socket;
   }

   public static ServerSocketFactory getDefault() {
      DomainServerSocketFactory ssf = null;

      try {
         InitialContext e = new InitialContext();
         SecurityDomain sd = (SecurityDomain)e.lookup("java:/jaas/other");
         ssf = new DomainServerSocketFactory(sd);
      } catch (Exception var3) {
         log.error("Failed to create default ServerSocketFactory", var3);
      }

      return ssf;
   }

   public String[] getDefaultCipherSuites() {
      String[] cipherSuites = new String[0];

      try {
         this.initSSLContext();
         SSLServerSocketFactory e = this.sslCtx.getServerSocketFactory();
         cipherSuites = e.getDefaultCipherSuites();
      } catch (IOException var3) {
         log.error("Failed to get default SSLServerSocketFactory", var3);
      }

      return cipherSuites;
   }

   public String[] getSupportedCipherSuites() {
      String[] cipherSuites = new String[0];

      try {
         this.initSSLContext();
         SSLServerSocketFactory e = this.sslCtx.getServerSocketFactory();
         cipherSuites = e.getSupportedCipherSuites();
      } catch (IOException var3) {
         log.error("Failed to get default SSLServerSocketFactory", var3);
      }

      return cipherSuites;
   }

   private void initSSLContext() throws IOException {
      if(this.sslCtx == null) {
         this.sslCtx = Context.forDomain(this.securityDomain);
      }
   }

}
